# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_unified_test.proto
# plugin: python-betterproto

from dataclasses import dataclass

import betterproto


@dataclass
class CMsgTest_MessageToClient_Request(betterproto.Message):
    some_text: str = betterproto.string_field(1)


@dataclass
class CMsgTest_MessageToClient_Response(betterproto.Message):
    some_text: str = betterproto.string_field(1)


@dataclass
class CMsgTest_NotifyClient_Notification(betterproto.Message):
    some_text: str = betterproto.string_field(1)


@dataclass
class CMsgTest_MessageToServer_Request(betterproto.Message):
    some_text: str = betterproto.string_field(1)


@dataclass
class CMsgTest_MessageToServer_Response(betterproto.Message):
    some_text: str = betterproto.string_field(1)


@dataclass
class CMsgTest_NotifyServer_Notification(betterproto.Message):
    some_text: str = betterproto.string_field(1)
