# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_video.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto


@dataclass
class CVideo_ClientGetVideoURL_Request(betterproto.Message):
    video_id: int = betterproto.uint64_field(1)
    client_cellid: int = betterproto.uint32_field(2)


@dataclass
class CVideo_ClientGetVideoURL_Response(betterproto.Message):
    video_id: int = betterproto.uint64_field(1)
    video_url: str = betterproto.string_field(2)


@dataclass
class VideoBookmark(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    playback_position_in_seconds: int = betterproto.uint32_field(2)
    video_track_id: int = betterproto.uint64_field(3)
    audio_track_id: int = betterproto.uint64_field(4)
    timedtext_track_id: int = betterproto.uint64_field(5)
    last_modified: int = betterproto.uint32_field(6)
    hide_from_watch_history: bool = betterproto.bool_field(7)
    hide_from_library: bool = betterproto.bool_field(8)


@dataclass
class CVideo_SetVideoBookmark_Notification(betterproto.Message):
    bookmarks: List["VideoBookmark"] = betterproto.message_field(1)


@dataclass
class CVideo_GetVideoBookmarks_Request(betterproto.Message):
    appids: List[int] = betterproto.uint32_field(1)
    updated_since: int = betterproto.uint32_field(2)


@dataclass
class CVideo_GetVideoBookmarks_Response(betterproto.Message):
    bookmarks: List["VideoBookmark"] = betterproto.message_field(1)


@dataclass
class CVideo_UnlockedH264_Notification(betterproto.Message):
    encryption_key: bytes = betterproto.bytes_field(1)


@dataclass
class CFovasVideo_ClientGetOPFSettings_Request(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    client_cellid: int = betterproto.uint32_field(2)


@dataclass
class CFovasVideo_ClientGetOPFSettings_Response(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    opf_settings: str = betterproto.string_field(2)
